from datetime import datetime, timedelta
from logging.handlers import TimedRotatingFileHandler
import os
import time

class DailyRotatingFileHandler(TimedRotatingFileHandler):
    def __init__(self, filename,file_type='log', when='midnight', interval=1, backupCount=5, encoding=None, delay=False, utc=False,maxBytes=10485760,mutiple_process=False):
        # 获取基础文件名（不带日期）
        self.base_filename = filename
        self.file_type = file_type
        # 初始文件名包含当前日期
        self.suffix = "%Y%m%d"
        self.mutiple_process = mutiple_process
        self.baseFilename = self._get_filename()
        self.maxBytes = maxBytes
        self.tommorw = self.get_next_midnight_timestamp()
        super().__init__(
            filename=self.baseFilename,
            when=when,
            interval=interval,
            backupCount=backupCount,
            encoding=encoding,
            delay=delay,
            utc=utc
        )
    def get_next_midnight_timestamp(self):
        # 获取当前时间
        now = datetime.now()
        
        # 计算明天的日期
        next_day = now + timedelta(days=1)
        
        # 设置时间为明天的0点0分0秒
        next_midnight = datetime(next_day.year, next_day.month, next_day.day, 0, 0, 0)
        
        # 转换为时间戳
        next_midnight_timestamp = int(next_midnight.timestamp())
        
        return next_midnight_timestamp
    def _get_base_file_name(self):
        dt = time.strftime('%Y%m%d')
        if self.mutiple_process:
            return f"{self.base_filename}{os.getpid()}.{dt}"
        else:
            return f"{self.base_filename}.{dt}"
    def _get_filename(self):
        return f'{self._get_base_file_name()}.0.{self.file_type}'
    def shouldRollover(self, record):
        # 检查文件大小或日期变更
        if self.stream is None:
            self.stream = self._open()
        if self.maxBytes > 0:
            msg = "%s\n" % self.format(record)
            self.stream.seek(0, 2)  # 文件末尾
            if self.stream.tell() + len(msg) >= self.maxBytes:
                return 1
        current_time = int(time.time())
        if current_time >= self.tommorw:
            return 1
        return 0
    def DeleteFiels(self):
        dirName, baseName = os.path.split(self.base_filename)
        fileNames = os.listdir(dirName)
        if len(fileNames)<self.backupCount:
            return
        # 过滤出符合模式的文件
        matching_files = [f for f in fileNames if f.startswith(baseName) and f.endswith(f".{self.file_type}")]
        
        # 按时间排序，最新的文件在最后
        matching_files.sort(key=lambda x: os.path.getmtime(os.path.join(dirName, x)))
        
        # 计算需要保留的文件数量
        files_to_keep = matching_files[-self.backupCount:]
        
        # 删除不需要的文件
        for f in matching_files:
            if f not in files_to_keep:
                os.remove(os.path.join(dirName, f))
          
    def doRollover(self):
        # 处理日期变更和文件大小限制
        if self.stream:
            self.stream.close()
            self.stream = None

        current_time = int(time.time())
        if current_time>=self.tommorw:
            self.baseFilename = self._get_filename()
            self.tommorw = self.get_next_midnight_timestamp()
            self.base_dir = os.path.dirname(self.baseFilename)
            os.makedirs(self.base_dir, exist_ok=True)
        else:
            cnt = int(self.baseFilename.split(".")[-2:][0])
            cnt +=1
            baseFile = self._get_base_file_name()
            dfn = f"{baseFile}.{cnt}.{self.file_type}"
            if os.path.exists(dfn):
                cnt += 1
                while True:
                    dfn = f"{baseFile}.{cnt}.{self.file_type}"
                    if not os.path.exists(dfn):
                        break
                    cnt += 1
            self.baseFilename = dfn
        if self.backupCount > 0:
            self.DeleteFiels()
        if not self.delay:
            self.stream = self._open()
        
        # 更新下次滚动时间
        new_rollover_at = self.computeRollover(current_time)
        while new_rollover_at <= current_time:
            new_rollover_at += self.interval
        
        self.rolloverAt = new_rollover_at


from contextvars import ContextVar
import time


trace_id = ContextVar("trace_id", default='')
session_id = ContextVar("session_id", default='')
seq = ContextVar("seq", default=0)
user_id = ContextVar("user_id", default='0')
user_info = ContextVar("user_info", default={})
current_tenant_id = ContextVar("current_tenant_id", default=0)
tenant_ids = ContextVar("tenant_ids", default=[])
dep_id = ContextVar("dep_id", default=0)
kxy_roles = ContextVar("roles", default=[])
access_token = ContextVar("access_token", default='')
last_log_time = ContextVar("last_log_time", default=time.time())



class DeleteSafeList():
    def __init__(self, list):
        self.list = list
        self.cursor = len(list)
    
    def __iter__(self):
        self.cursor = len(self.list)
        return self
    
    def __next__(self):
        if self.cursor <= 0:
            self.cursor = len(self.list)
            raise StopIteration
        self.cursor -= 1
        return self.list[self.cursor]
    def RemoveCurrent(self):
        self.list.pop(self.cursor)
    def __len__(self):
        return len(self.list)
        
        
    # def remove(self, value):
    #     current_cursor=-1
    #     while True:
    #         if current_cursor < 0:
    #             break
    #         if self.list[self.cursor] == value:
    #             self.list.pop(self.cursor)
    #         else:
    #             break
    #     self.cursor -= 1


import asyncio
import random
import sys
import os

# 正确添加项目根路径
project_root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
if project_root not in sys.path:
    sys.path.insert(0, project_root)
    
from kxy.framework.context import user_id, user_info, current_tenant_id, dep_id
from datetime import datetime

user_id.set('43')
current_tenant_id.set(1)
dep_id.set(0)
user_info.set({'chineseName':'张三'})

from sqlalchemy.ext.asyncio import create_async_engine, AsyncSession
from sqlalchemy.orm import sessionmaker, declarative_base
from test.config import config

engine = create_async_engine(config.mysql_url, pool_size=5,pool_pre_ping=True,pool_recycle=1800, max_overflow=20,echo=True)
AsyncSessionLocal = sessionmaker(engine, class_=AsyncSession, expire_on_commit=False,autoflush=False,autocommit=False)

import logging
from kxy.framework.slogger import create_logger,new_trace
logger,handler = create_logger(logging.DEBUG,'appName','production',filename='log/app',file_type='log',backupCount=5,maxBytes=10485760,mutiple_process=False)
new_trace()
logger.info('hello world','grpc')


async def get_db():
    async with AsyncSessionLocal() as session:
        yield session
        
from test.test_table_dal import TestTableDal
from test.test_table import TestTable

async def test_table_add():
    async with AsyncSessionLocal() as session:
        dal = TestTableDal(session)
        for i in range(20,30):
            tab = TestTable()
            tab.user_id=user_id.get()
            tab.UserName = f'张三{i}'
            tab.Sex =1
            tab.ExpiresTime =datetime.now()
            tab.IsDelete = 0
            tab.TenantId = i%3
            tab.DepId=i%3
            await dal.Insert(tab)
async def query():
    async with AsyncSessionLocal() as session:
        dal = TestTableDal(session)
        tab = TestTable()
        tab.user_id=user_id.get()
        tab.UserName = f'张三{random.randint(1,1000)}'
        tab.Sex =1
        tab.ExpiresTime =datetime.now()
        tab.IsDelete = 0
        # tab.TenantId = 1
        tab.DepId=1
        await dal.Insert(tab)
        logger.info(f'插入数据成功,id为{tab.Id}','sql')
        
        exist =await dal.Get(tab.Id)
        assert exist is not None , '查询失败'
        
        result = await dal.ExecSql('select * from test_table')
        items = result.all()
        test_table_objects = [TestTable().init_with_dict(row._asdict()) for row in items]
        
        assert len(test_table_objects) > 0, '自定义查询失败'

        results,total = await dal.Search({},1,30)
        print(total,[res.Id for res in results])

        data = await dal.Get(results[0].Id)
        print(data.toDic())

        # await dal.Delete(data.Id)
        await dal.GetCustomer(data.Id)
        await dal.GetCustomerNoFilter(data.Id)
        total = await dal.DeleteWhere([TestTable.Id==data.Id])
        print(f'删除了{total}条数据')
        assert total == 1,'删除结果不为1'

loop:asyncio.AbstractEventLoop = asyncio.new_event_loop()
# loop.run_until_complete(test_table_add())
loop.run_until_complete(query())


class Mapper:
    __mapper_relation = {}
    logger=None
    @staticmethod
    def register(cls):
        name=cls.__name__
        if Mapper.__mapper_relation.get(name) is None:
            Mapper.__mapper_relation[cls.__name__] = cls

    @staticmethod
    def getservice(clsName,*arg,**agrs):
        return Mapper.__mapper_relation[clsName](*arg,**agrs)

    @staticmethod
    def getclass(clsName):
        return Mapper.__mapper_relation[clsName]


import json
import logging
import os
import time
import uuid

from .util import SUtil
from .DailyRotatingFileHandler import DailyRotatingFileHandler
from .context import trace_id,session_id,seq,user_id,last_log_time

class TraceIdFilter(logging.Filter):
    def filter(self, record):
        record.trace_id = trace_id.get()
        record.session_id = session_id.get()
        record.userid = user_id.get()
        cur_seq = seq.get()
        record.seq = cur_seq
        seq.set(cur_seq+1)
        return True
def new_trace(traceId=''):
    if not traceId:
        traceId = uuid.uuid4().hex[:16]
    trace_id.set(traceId)
    session_id.set(uuid.uuid4().hex[:16])
    seq.set(0)

from logging import Logger
class slogger(Logger):
    def __init__(self, logger:Logger, level=0):
        super().__init__(logger.name,logger.level)
        self.logger = logger
        self.parent = logger.parent
        self.propagate = logger.propagate
        self.handlers = logger.handlers
        self.disabled = logger.disabled
    def _add_log_category(self, log_method, msg, logCategory='default', *args, **kwargs):
        """通用的日志分类添加方法"""
        extra = kwargs.get('extra', {})
        extra["logCategory"] = logCategory
        kwargs['extra'] = extra
        return log_method(msg, *args, **kwargs)
    
    def info(self, msg, logCategory='default', *args, **kwargs):
        return self._add_log_category(self.logger.info, msg, logCategory, *args, **kwargs)
    
    def debug(self, msg, logCategory='default', *args, **kwargs):
        return self._add_log_category(self.logger.debug, msg, logCategory, *args, **kwargs)
    
    def warning(self, msg, logCategory='default', *args, **kwargs):
        return self._add_log_category(self.logger.warning, msg, logCategory, *args, **kwargs)
    
    def error(self, msg, logCategory='default', *args, **kwargs):
        return self._add_log_category(self.logger.error, msg, logCategory, *args, **kwargs)
    
    def critical(self, msg, logCategory='default', *args, **kwargs):
        return self._add_log_category(self.logger.critical, msg, logCategory, *args, **kwargs)
    
    def exception(self, msg, logCategory='default', *args, **kwargs):
        return self._add_log_category(self.logger.exception, msg, logCategory, *args, **kwargs)

def create_logger(logLeve,appName,env,filename='log/app',file_type='log',backupCount=5,maxBytes=10485760,mutiple_process=False):
    localIp = SUtil.get_local_ip()
    class JsonFormatter(logging.Formatter):
        def format(self, record):
            # 计算与上次日志的时间间隔
            current_time = time.time()
            prev_time = last_log_time.get()
            duration = current_time - prev_time if prev_time > 0 else 0
            last_log_time.set(current_time)
            
            # 构建日志记录的字典
            log_record = {
                "appName":appName,
                "serverAddr":os.environ.get('IP',localIp),
                "cluster": env,
                "level": record.levelname,
                "filename": record.filename,
                "lineno": record.lineno,
                "traceId": record.trace_id,
                "sessionId": record.session_id,
                "userId":record.userid,
                "seqId": record.seq,
                "message": record.getMessage(),
                "CreateTime": self.formatTime(record, self.datefmt),
                "createdOn": int(time.time() * 1000),  # 添加 Unix 时间戳
                "duration": int(duration * 10000000)  # 添加持续时间，单位毫秒，保留2位小数
            }
            if hasattr(record, 'logCategory'):
                log_record["logCategory"] = record.logCategory
            else:
                log_record["logCategory"] = "Default"
            # 将字典转换为 JSON 字符串
            return json.dumps(log_record, ensure_ascii=False)
        
    logging.basicConfig(
        level=logLeve
    )
    _logger = logging.getLogger(appName)
    _logger.setLevel(logLeve)
    # 创建一个 RotatingFileHandler 对象
    # 确保 log 目录存在
    if '/' in filename:
        floder = filename.split('/')[0]
        os.makedirs(floder, exist_ok=True)
    handler = DailyRotatingFileHandler(
        filename=filename,
        file_type=file_type,
        when='midnight',
        interval=300,
        backupCount=backupCount,
        maxBytes=maxBytes,  # 10MB
        mutiple_process = mutiple_process
    )
    formatter = JsonFormatter()

    # 设置日志记录级别
    handler.setLevel(logLeve)
    handler.setFormatter(formatter)
    handler.addFilter(TraceIdFilter())

    _logger.addHandler(handler)
    logger = slogger(_logger)
    return logger,handler


import os
from abc import ABC, abstractmethod

class BaseConfig(ABC):
    BussinessLog = False
    @abstractmethod
    def SSO_URL(self):
        pass

    @abstractmethod
    def SystemCode(self):
        pass
    @abstractmethod
    def JWT_SECRET_KEY(self):
        pass
    @abstractmethod
    def JWT_ALGORITHM(self):
        pass

    def env_first(self):
        for name in [f for f in dir(self) if not callable(f) and not f.startswith('__')]:
            v=os.environ.get(name)
            if v:
                setattr(self, name, v)


from datetime import datetime, timedelta
import time
from typing import Tuple

class DateUtil():
    @staticmethod
    def now_timestamp()->float:
        '''获取当前时间戳'''
        return time.time()
    
    @staticmethod
    def convert_timestamp_to_datetime(timestamp:int)->datetime:
        '''时间戳转时间'''
        # 判断时间戳单位：如果大于1e10，则认为是毫秒单位
        if timestamp > 1e10:
            # 毫秒单位，需要转换为秒
            timestamp_seconds = timestamp / 1000
        else:
            # 秒单位，直接使用
            timestamp_seconds = timestamp
        
        return datetime.fromtimestamp(timestamp_seconds)
    @staticmethod
    def is_expired_timestamp(timestamp)->bool:
        '''判断时间戳是否过期'''
        return DateUtil.now_timestamp() > timestamp
    @staticmethod
    def is_expired_date(date:datetime)->bool:
        '''判断时间是否过期'''
        return DateUtil.is_expired_timestamp(date.timestamp())
    
    @staticmethod
    def timestamp_range_str(begin_time_str,end_time_str)->Tuple[float,float]:
        date_format = "%Y-%m-%d %H:%M:%S"
        start_time = datetime.strptime(begin_time_str, date_format)
        end_time = datetime.strptime(end_time_str, date_format)
        return DateUtil.timestamp_range(start_time,end_time)
    @staticmethod
    def timestamp_range(start_time:datetime,end_time:datetime)->Tuple[float,float]:
        # 转换为时间戳
        start_timestamp = int(start_time.timestamp())
        end_timestamp = int(end_time.timestamp())
        return start_timestamp,end_timestamp
    @staticmethod
    def timestamp_from_now(day=0,hours=0,minutes=0,seconds=0)->Tuple[float,float]:
        """计算从当前时间到未来时间，返回时间戳(精确到秒)，如果传入负数，则返回从过去到当前的时间戳

        Args:
            day (int, optional): 天. Defaults to 0.
            hours (int, optional): 小时. Defaults to 0.
            minutes (int, optional): 分钟. Defaults to 0.
            seconds (int, optional): 秒. Defaults to 0.

        Returns:
            Tuple[float,float]: 开始时间戳，结束时间戳
        """        
        now =datetime.now()
        end = now + timedelta(days=day,hours=hours,minutes=minutes,seconds=seconds)
        now_t = int(now.timestamp())
        end_t = int(end.timestamp())
        if now_t > end_t:
            return end_t,now_t
        return now_t,end_t
    @staticmethod
    def get0230(day:datetime)->Tuple[datetime,datetime]:
        begin = datetime(day.year, day.month, day.day,hour=0,minute=0,second=0)
        end = datetime(day.year, day.month, day.day,hour=23,minute=59,second=59)
        return begin,end
        
    @staticmethod
    def datetime_range(day:datetime,beginHour=22,beginMinits=0,willHour=0) -> Tuple[datetime,datetime]:
        """获取指定日期的指定小时到指定小时的时间，如果endHour小于beginHour，则结束时间为第二天

        Args:
            day (datetime): 日期
            beginHour (int, optional): 开始小时. Defaults to 22.
            beginMinits (int, optional): 开始分钟. Defaults to 0.
            willHour (int, optional): 未来偏移量小时. Defaults to 8.

        Returns:
            Tuple[datetime.datetime,datetime.datetime]: _description_
        """        
        begin=datetime(day.year, day.month, day.day,hour=beginHour,minute=beginMinits)
        end=None
        if willHour:
            end=begin+timedelta(hours=willHour)
        else:
            end = begin
        if end<begin:
            return end,begin
        return begin,end
        
        
if __name__ == '__main__':
    # print(DateUtil.timestamp_from_now(minutes=-5))
    print(DateUtil.getRangeDate(datetime.now(),beginHour=22,willHour=10))